# Resolve a topic

Zulip's [topics](/help/streams-and-topics) are very
helpful for customer support, answering questions, investigating
issues and production errors, and other workflows.

Resolving topics makes it easy to track the status of each question,
investigation, or notification. Marking a topic as resolved:

* Puts a ✔ at the beginning of the topic name, e.g. `example topic`
  becomes `✔ example topic`.
* Triggers an automated message from Notification Bot indicating that
  you resolved the topic. This message will be marked as unread
  only for users who had participated in the topic.
* Changes whether the topic appears when using the `is:resolved` and
  `-is:resolved` [search operators](/help/search-for-messages).

Marking a topic as unresolved removes the ✔ and also triggers a
Notification Bot message.

It's often helpful to define a policy for when to resolve topics that
fits how topics are used in a given stream. Here are some common
approaches for deciding when to mark a topic as resolved:

* **Support**: When the support interaction is complete. Resolving
  topics is particularly useful for internal support teams that might
  not need a dedicated support ticket tracker.
* **Issues, errors and production incidents**: When investigation or
  incident response is complete, and any follow-up work has been
  transferred to the appropriate tracker.
* **Workflow management**: What the work described in the topic is
  complete and any follow-ups have been transcribed.
* **Answering questions**: When the question has been fully answered,
  and follow-ups would be best discussed in a new topic.

Users can resolve and unresolve a topic if they have permission to edit
topics. Organization administrators can [configure who can edit
topics](/help/configure-who-can-edit-topics) or turn off message
editing entirely. See the [guide to message and topic
editing](/help/configure-message-editing-and-deletion) for details
on when topic editing is allowed.

## Mark a topic as resolved

### Via the message recipient bar

{start_tabs}

1. Click on the **✔** icon to mark an unresolved topic as resolved.

{end_tabs}

### Via the left sidebar

{start_tabs}

{!topic-actions.md!}

1. Select **Mark as resolved**.

{end_tabs}

## Mark a topic as unresolved

### Via the message recipient bar

{start_tabs}

1. Click on the **✔** icon to mark a resolved topic as unresolved.

{end_tabs}

### Via the left sidebar

{start_tabs}

{!topic-actions.md!}

1. Select **Mark as unresolved**.

{end_tabs}


## Details

* Resolving a topic works by moving the messages to a new topic.
* Like with all topic editing, Zulip clients update instantly, so
  human users will likely only send messages to the resolved topic.
* [Integrations](/integrations) will usually send new messages to the
  original topic (`example topic`) after a topic is resolved. This is
  useful for alerting integrations, where a repeating alert might have a
  different cause. You can mark the topic resolved (as normal) once
  you've investigated the situation.
* Users can still send messages to a resolved topic; this
  is important for _"thank you"_ messages and to discuss whether
  the topic was incorrectly marked as resolved.

## Related articles

* [Rename a topic](/help/rename-a-topic)
* [Move content to another topic](/help/move-content-to-another-topic)
* [Restrict topic editing](/help/configure-who-can-edit-topics)
* [Configure message editing and deletion](/help/configure-message-editing-and-deletion)
* [API documentation for resolving topics](/api/update-message)
